#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


/* compile with 
   gcc -g -Wall -O3 matrix_multiply.c -o matrix_multiply
*/

double * matrix_load(const char * fname, int *n_rows, int *n_cols) {
  FILE *f = fopen(fname, "r");
  if(!f) {
    fprintf(stderr, "cannot open %s", fname); perror("");
    exit(1);
  }
  int size[2];
  int ret = fread(size, sizeof(int), 2, f);
  assert(ret == 2);
  *n_rows = size[0]; *n_cols = size[1];
  double *data = malloc(sizeof(double) * size[0] * size[1]); 
  ret = fread(data, sizeof(double), size[0] * size[1], f);
  assert(ret == size[0] * size[1]); 
  fclose(f);
  return data;
}

void matrix_store(const char * fname, int n_rows, int n_cols, const double *data) {
  FILE *f = fopen(fname, "w");
  if(!f) {
    fprintf(stderr, "cannot open %s", fname); perror("");
    exit(1);
  }
  int size[2] = {n_rows, n_cols};
  int ret = fwrite(size, sizeof(int), 2, f);
  assert(ret == 2);
  ret = fwrite(data, sizeof(double), size[0] * size[1], f);
  assert(ret == size[0] * size[1]); 
  fclose(f);
}






int main(int argc, char ** argv) {
  /* compute C := A * B */
  int m;  /* number of rows of the matrix A and of the matrix C */
  int n;  /* number of columns of the matrix B and the number of columns of the matrix C */
  int k;  /* number of columns of the matrix A and the number of rows of the matrix  B */

  assert(argc == 4); 
  
  const char *a_filename = argv[1]; 
  const char *b_filename = argv[2]; 
  const char *c_filename = argv[3]; 
  
  double * a = matrix_load(a_filename, &m, &k); 

  int k2; 
  double * b = matrix_load(b_filename, &k2, &n); 

  printf("Matrix A: %d * %d, B: %d * %d\n", m, k, k2, n); 

  assert(k == k2 || !"matrix dimensions not compatible");

  double *c = malloc(sizeof(double) * m * n); 

#define A(i, j) a[(i) * (k) + (j)]
#define B(i, j) b[(i) * (n) + (j)]
#define C(i, j) c[(i) * (n) + (j)]
  
  int i, j, l; 
  
  for(i = 0; i < m; i++) {
    for(j = 0; j < n; j++) {
      double accu = 0;
      for(l = 0; l < k; l++)
	accu += A(i, l) * B(l , j);
      C(i, j) = accu;
    }
  }
  
  matrix_store(c_filename, m, n, c);

  free(a);
  free(b);
  free(c);
  
  return 0;
}

