function circle_ind = affine_adaptation_test(I, circles)

global max_aspect_ratio 

num_pts = size(circles, 1);

%%
%% compute second moment matrices for selected points at characteristic scales
%%
[GX,GY] = gen_dgauss(2);
I_X = filter2(GX, I, 'same');
I_Y = filter2(GY, I, 'same');
I_XX = I_X .* I_X;
I_XY = I_X .* I_Y;
I_YY = I_Y .* I_Y;

delete_flag = zeros(num_pts,1);

fprintf('Performing affine adaptation...\n');
for i=1:num_pts
%     if mod(i,100)==0
%         fprintf('***** Point %d, total: %d\n', i, num_pts);
%     end

    G = gen_gauss(circles(i,3)*1.2);
    f_wid = floor((size(G,1) - 1)/2);
    cx = circles(i,1);
    cy = circles(i,2);
    
    %%
    %% compute second moment matrix over Gaussian neighborhood
    %%
    V_XX = filter_point(G, I_XX, cx, cy);
    V_XY = filter_point(G, I_XY, cx, cy);
    V_YY = filter_point(G, I_YY, cx, cy);
    M = [V_XX V_XY; V_XY V_YY];

    % get affine transformation
    [R1,D] = eig(M);
    
    D = diag(diag(D).^(-1/2));
    D = D * (2 * circles(i,3))/sum(diag(D));
    % transformation from unit circle to ellipse: Q = R1 * D; 
        
    % check aspect ratio
    d1 = max(D(1,1),D(2,2));
    d2 = min(D(1,1),D(2,2));
    if (d2 < eps) | ((d1/d2) > max_aspect_ratio)
        delete_flag(i) = 1;
    end
end

circle_ind = find(delete_flag==0);
