function [scale_space, scale_space2] = build_scale_spaces(edge_arr, hgt, wid)
%%
%% Build scale spaces of cirle responses: each location (x,y,r) contains the strength of response for circle with center (x,y) and radius r 
%% scale_space: each edge point contributes a weighted vote for a given circle, i.e., the vote is equal to the magnitude of the edge strength at that point
%% scale_space2: each edge point contributes an unweighted vote of 1
%%

global sigma_edge_factor sigma_edge_init sigma_edge_final max_sigma_r
global r_init r_final r_iters r_factor r_final_factor sigma_smear max_filter_width

if r_final_factor > 0
	r_final = sqrt(wid^2 + hgt^2) * r_final_factor;
end
r_iters = round(log(r_final / r_init) / log(r_factor));

scale_space = zeros(hgt, wid, r_iters);     % weighted  
scale_space2 = zeros(hgt, wid, r_iters);    % unweighted

r = r_init;
sigma_edge = sigma_edge_init;

for ri = 1:r_iters

        fprintf('scale %d/%d\r', ri, r_iters);
  
	%% create the gaussian filter
	sigma_r = min(0.75 + sigma_smear * (r - r_init), max_sigma_r);
	%    sigma_r = min(sigma_smear * r, max_sigma_r);
	f_wid = round(8 * sigma_r);
	if mod(f_wid,2) == 0
		f_wid = f_wid + 1;
	end
	f_wid = min(f_wid, max_filter_width);
	
	se = min(sigma_edge_final, floor(sigma_edge));
	
	F = fspecial('gaussian', f_wid, sigma_r);
	F = F ./ max(F(:)); % 1 in the center
	
	curr_scale_space = zeros(hgt,wid);
	curr_scale_space2 = zeros(hgt,wid);
	
	cx1_arr = round(edge_arr{se}(:,1) + edge_arr{se}(:,3) .* r);
	cy1_arr = round(edge_arr{se}(:,2) + edge_arr{se}(:,4) .* r);
	cx2_arr = round(edge_arr{se}(:,1) - edge_arr{se}(:,3) .* r);
	cy2_arr = round(edge_arr{se}(:,2) - edge_arr{se}(:,4) .* r);
	
	% find out of bounds indices
	delete_flag = (cx1_arr < 1) | (cx1_arr > wid) | (cy1_arr < 1) | (cy1_arr > hgt);
	delete_ind = find(delete_flag);
	
	for ii = 1:size(cx1_arr,1)
		if delete_flag(ii) % out of bounds
			continue;
		end
		% increment by 1 or by edge strength
		curr_scale_space(cy1_arr(ii), cx1_arr(ii)) = curr_scale_space(cy1_arr(ii), cx1_arr(ii)) + edge_arr{se}(ii, 5);
		curr_scale_space2(cy1_arr(ii), cx1_arr(ii)) = curr_scale_space(cy1_arr(ii), cx1_arr(ii)) + 1;
	end
	
	delete_flag = (cx2_arr < 1) | (cx2_arr > wid) | (cy2_arr < 1) | (cy2_arr > hgt);
	delete_ind = find(delete_flag);
	
	for ii = 1:size(cx2_arr,1)
		if delete_flag(ii)
			continue;
		end
		curr_scale_space(cy2_arr(ii), cx2_arr(ii)) = curr_scale_space(cy2_arr(ii), cx2_arr(ii)) + edge_arr{se}(ii, 5);
		curr_scale_space2(cy2_arr(ii), cx2_arr(ii)) = curr_scale_space(cy2_arr(ii), cx2_arr(ii)) + 1;
	end
	
    % blur the scale spaces
	curr_scale_space = filter2(F, curr_scale_space, 'same');
	curr_scale_space2 = filter2(F, curr_scale_space2, 'same');
	
    % normalization by circumference
	scale_space2(:,:,ri) = curr_scale_space2 ./ (2 * pi * r);              
	scale_space(:,:,ri) = curr_scale_space ./ (2 * pi * r);        
	r = r * r_factor;
	sigma_edge = sigma_edge * sigma_edge_factor;
end    % ri

fprintf('\n');
