function circles = find_maxima_double(I, scale_space, scale_space2)
%%
%% Find maxima in scale_space (weighted circle responses) and use scale_space2 (unweighted responses) for additional thresholding
%%

global r_init r_final r_iters r_factor sigma_edge_init sigma_edge_final sigma_edge_factor
global nonmax_wid max_points unweighted_thresh_rel unweighted_thresh_abs border_width max_aspect_ratio

nonmax_size = [nonmax_wid nonmax_wid];
hgt = size(scale_space,1);
wid = size(scale_space,2);

% stage 1: nonmaximum suppression separately at each scale level
circles = zeros(0,6);
for ri = 1:r_iters
 	M_suppress = colfilt(scale_space(:,:,ri), nonmax_size, 'sliding', @nonmax_suppress_col);        
    scale_space(:,:,ri) = M_suppress;
end

% stage 2: suppress non-maxima across neighboring scales
r = r_init;
sigma_edge = sigma_edge_init;
for ri = 1:r_iters

    curr_scale_space = scale_space(:,:,ri);       
    curr_scale_space2 = scale_space2(:,:,ri);
    thresh_flag = (curr_scale_space2 >= unweighted_thresh_rel) & ((curr_scale_space2 .* (2 * pi * r)) > unweighted_thresh_abs);
    curr_scale_space = curr_scale_space .* thresh_flag;
    [i_ind j_ind] = find(curr_scale_space);
    
    if (r - r / r_factor) < nonmax_wid
        for ii = 1:size(i_ind,1)
            
            cx = j_ind(ii);
            cy = i_ind(ii);
            
            x_lo = cx - floor(nonmax_wid/2);
            x_hi = cx + floor(nonmax_wid/2);
            y_lo = cy - floor(nonmax_wid/2);
            y_hi = cy + floor(nonmax_wid/2);                            
            x_lo = max(1, x_lo);
            x_hi = min(size(scale_space2,2), x_hi);
            y_lo = max(1, y_lo);
            y_hi = min(size(scale_space2,1), y_hi);
            
            ri_iters = round(log((r + nonmax_wid) / r) / log(r_factor));
            ri_lo = max(1, ri - ri_iters);
            ri_hi = min(r_iters, ri + ri_iters);
            
            if curr_scale_space(cy,cx) < max(max(max(scale_space(y_lo:y_hi,x_lo:x_hi,ri_lo:ri_hi))))
                curr_scale_space(cy,cx) = 0;
            end
        end
    end
    [i_ind j_ind] = find(curr_scale_space);
    if ~isempty(i_ind)
        counts = curr_scale_space2(find(curr_scale_space));
        counts = counts .* (2 * pi * r);
        weights = curr_scale_space(find(curr_scale_space));
        weights = weights .* (2 * pi * r);
        se = min(floor(sigma_edge), sigma_edge_final);
        circles = [circles; j_ind i_ind ones(size(i_ind)) * r weights counts ones(size(i_ind)) * se]; % [x y radius weight support sigma_edge]
    end
    r = r * r_factor;
    sigma_edge = sigma_edge * sigma_edge_factor;
end

% remove all circles that touch the border
border_arr = [circles(:,1) + circles(:,3) circles(:,1) - circles(:,3) circles(:,2) + circles(:,3) circles(:,2) - circles(:,3)];
remove_pts = (border_arr(:,1) >= wid - border_width) | (border_arr(:,2) < border_width) | (border_arr(:,3) >= hgt - border_width) | (border_arr(:,4) < border_width);
circles(find(remove_pts),:) = [];

% perform affine adaptation test (remove circles with degenerate second moment matrices)
ind = affine_adaptation_test(I, circles);
circles = circles(ind,:);

circles = flipud(sortrows(circles, 4));

if size(circles,1) > max_points
    circles = circles(1:max_points,:);
end
