function E = gen_ellipse_mask(D,R,enlarge_factor)
% D is a diagonal matrix
% R is a forward rotation matrix

if nargin < 3
    enlarge_factor = 3;
end

sigma1 = D(1,1);
sigma2 = D(2,2);

f_wid_x = enlarge_factor * floor(sigma1);
f_wid_y = enlarge_factor * floor(sigma2);

% find the extent of the grid for the transformed Gaussian
corners = [ -f_wid_x,  f_wid_x, f_wid_x, -f_wid_x; 
   f_wid_y, -f_wid_y, f_wid_y, -f_wid_y ];
new_corners = R * corners;
x_min = floor(min(new_corners(1,:)));
x_max = ceil(max(new_corners(1,:)));
y_min = floor(min(new_corners(2,:)));
y_max = ceil(max(new_corners(2,:)));
[grid_x_t, grid_y_t] = meshgrid(x_min:x_max,y_min:y_max);

% ellipse matrix
Sinv = inv((R * D) * (R * D)');
dist_vec = Sinv(1,1) .* grid_x_t .* grid_x_t + ...
    Sinv(1,2) .* grid_x_t .* grid_y_t + Sinv(2,1) .* grid_x_t .* grid_y_t + ...
    Sinv(2,2) .* grid_y_t .* grid_y_t;
E = (dist_vec <= enlarge_factor^2);
% E = filter2(fspecial('gauss',3,1),E,'same');