function [G,DG1,DG2]=gen_rotated_gauss(D,R)
% D is a diagonal matrix
% R is a forward rotation matrix

sigma1 = D(1,1);
sigma2 = D(2,2);

f_wid_x = 2 * floor(sigma1);
f_wid_y = 2 * floor(sigma2);

% find the extent of the grid for the transformed Gaussian
corners = [ -f_wid_x,  f_wid_x, f_wid_x, -f_wid_x; 
   f_wid_y, -f_wid_y, f_wid_y, -f_wid_y ];
new_corners = R * corners;
x_min = floor(min(new_corners(1,:)));
x_max = ceil(max(new_corners(1,:)));
y_min = floor(min(new_corners(2,:)));
y_max = ceil(max(new_corners(2,:)));
[grid_x_t, grid_y_t] = meshgrid(x_min:x_max,y_min:y_max);

% covariance matrix
Sinv = inv((R * D) * (R * D)');
dist_vec = Sinv(1,1) .* grid_x_t .* grid_x_t + ... 
    Sinv(1,2) .* grid_x_t .* grid_y_t + Sinv(2,1) .* grid_x_t .* grid_y_t + ...
    Sinv(2,2) .* grid_y_t .* grid_y_t;
G = exp(-0.5 * dist_vec);
G = G ./ sum(sum(G));

DGx = (-Sinv(1,1) .* grid_x_t - Sinv(1,2) .* grid_y_t) .* exp(-0.5 * dist_vec);
DGy = (-Sinv(2,2) .* grid_y_t - Sinv(1,2) .* grid_x_t) .* exp(-0.5 * dist_vec);

DG1 = R(1,1) * DGx + R(2,1) * DGy;
DG2 = R(2,2) * DGy + R(1,2) * DGx;
DG1 = DG1 * 2 ./ sum(sum(abs(DG1)));
DG2 = DG2 * 2 ./ sum(sum(abs(DG2)));
