function [G,L]=gen_rotated_gauss(D,R)
% D is a diagonal matrix
% R is a forward rotation matrix

sigma1 = D(1,1);
sigma2 = D(2,2);

f_wid_x = 3 * floor(sigma1);
f_wid_y = 3 * floor(sigma2);

% find the extent of the grid for the transformed Gaussian
corners = [ -f_wid_x,  f_wid_x, f_wid_x, -f_wid_x; 
   f_wid_y, -f_wid_y, f_wid_y, -f_wid_y ];
new_corners = R * corners;
x_min = floor(min(new_corners(1,:)));
x_max = ceil(max(new_corners(1,:)));
y_min = floor(min(new_corners(2,:)));
y_max = ceil(max(new_corners(2,:)));
[grid_x_t, grid_y_t] = meshgrid(x_min:x_max,y_min:y_max);

% back-transform the neighborhood and compute grid for non-rotated Gaussian
new_corners = [x_min, x_max, x_max, x_min; 
               y_min, y_max, y_min, y_max];
corners = R' * new_corners;
x_min = floor(min(corners(1,:)));
x_max = ceil(max(corners(1,:)));
y_min = floor(min(corners(2,:)));
y_max = ceil(max(corners(2,:)));

% make sure the dimensions are odd
if mod(x_max - x_min + 1,2)==0
    x_max = x_max + 1;
end
if mod(y_max - y_min + 1,2)==0
    y_max = y_max + 1;
end

[grid_x, grid_y] = meshgrid(x_min:x_max,y_min:y_max);

% generate non-rotated Gaussian and Laplacian
G_x = normpdf(x_min:x_max,0,sigma1);
G_y = normpdf(y_min:y_max,0,sigma2);
G = G_y' * G_x;
L = 4 * del2(G);

% rotate Gaussian and Laplacian
[grid_x_t, grid_y_t] = rotate_grid(grid_x_t, grid_y_t, R');

G = interp2(grid_x, grid_y, G, grid_x_t, grid_y_t);
L = interp2(grid_x, grid_y, L, grid_x_t, grid_y_t);

% plug the holes (for now)
G(find(isnan(G))) = 0;
L(find(isnan(L))) = 0;