%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%for i=1:5,
%    [contx,conty] = Smooth_Curve(contx,conty,3);
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [New_Cont_x, New_Cont_y] = smooth_curve(Cont_x, Cont_y,Sigma)

% Initialisations
New_Cont_x=zeros(size(Cont_x));
New_Cont_y=zeros(size(Cont_y));

n=size(Cont_x,1);

% Indexing by arc length
Arc_Length=zeros(n,1);
Arc_Length(1)=sqrt((Cont_x(1)-Cont_x(n))^2+(Cont_y(1)-Cont_y(n))^2);
for i=2:n
  Arc_Length(i)=sqrt((Cont_x(i)-Cont_x(i-1))^2+(Cont_y(i)-Cont_y(i-1))^2);
end;

% Finding the minimum value of the distances between 2 consecutive
% points gives the size of the rolling array
rho=min(Arc_Length);
if rho<1,
   rho=1;
end;
s=floor(4*Sigma/rho);

if s>n,
   error('Error : Smooth_Curve : size of the window is too high');
%   s=n;
end;

% Filling of the looping contour and the looping arc length arrays :
n_Prime=n+2*s;
Loop_Cont_x=zeros(n_Prime,1);
Loop_Cont_y=zeros(n_Prime,1);
Loop_Arc_Length=zeros(n_Prime,1);
Loop_Cum_Arc_Length=zeros(n_Prime,1);

Loop_Cont_x(1:s)=Cont_x((n-s+1):n);
Loop_Cont_x((s+1):(n_Prime-s))=Cont_x(1:n);
Loop_Cont_x((n_Prime-s+1):n_Prime)=Cont_x(1:s);

Loop_Cont_y(1:s)=Cont_y((n-s+1):n);
Loop_Cont_y((s+1):(n_Prime-s))=Cont_y(1:n);
Loop_Cont_y((n_Prime-s+1):n_Prime)=Cont_y(1:s);

Loop_Arc_Length(1:s)=Arc_Length((n-s+1):n);
Loop_Arc_Length((s+1):(n_Prime-s))=Arc_Length(1:n);
Loop_Arc_Length((n_Prime-s+1):n_Prime)=Arc_Length(1:s);

% Loop_Cum_Arc_Length contains a cumulative arc length
for i=2:n_Prime
  Loop_Cum_Arc_Length(i)=Loop_Arc_Length(i)+Loop_Cum_Arc_Length(i-1);
end;

% Main loop
for i=(s+1):(n_Prime-s)

  % filling the Rolling Array with relative distances
  Rolling_Array=Loop_Cum_Arc_Length((i-s):(i+s))- ...
      Loop_Cum_Arc_Length(i);

  % kiling out-of-zone points
  Bool_Array=(abs((Rolling_Array))<=(4*Sigma));


  % Computing the new values of x and y
  temparray=Bool_Array.*Gaussian(Rolling_Array,Sigma);
  temp_x=temparray.*Loop_Cont_x((i-s):(i+s));
  temp_y=temparray.*Loop_Cont_y((i-s):(i+s));
  dj=Rolling_Array(2:(2*s+1))-Rolling_Array(1:(2*s));

  weight=.5*sum(dj.*(temparray(2:(2*s+1))+temparray(1:(2*s))));

  New_Cont_x(i-s)=.5*sum(dj.*(temp_x(2:(2*s+1))+temp_x(1:(2*s))))/weight;
  New_Cont_y(i-s)=.5*sum(dj.*(temp_y(2:(2*s+1))+temp_y(1:(2*s))))/weight;

end;


function res=Gaussian(x,Sigma)
res=(1/(sqrt(2*pi)*Sigma))*exp(-(x.^2)/(2*Sigma^2));
